#include "header\gba.h"
#include "header\bg.h"
#include "header\timer.h"
#include "header\dma.h"
#include "header\irq.h"
#include "header\sound.h"
#include "header\sprite.h"
#include "header\blend.h"

#include "lib.h"
#include "iwram.h"
#include "irq.h"
#include "syscall.h"

#include "resource.h"
#include "resource\BGFont.h"
//#include "resource\BitMapFont.h"

//---------------------------------------------------------------------------
//^C̃TCY
#define BG_CX               16
#define BG_CY               16

//}bṽTCY
#define MAP_SIZE            8
#define MAP_CX              (MAP_SIZE+2)	//+2͊O̕ǔzup
#define MAP_CY              (MAP_SIZE+2)

//p}bv̒`
#define MAP_EMPTY           0
#define MAP_WHITE           1
#define MAP_BLACK           2
#define MAP_BLOCK           3

//\pBG^C
#define BG_TILE_EMPTY       2
#define BG_TILE_WHITE       4
#define BG_TILE_BLACK       6

//J[\ړ
#define UNIT_VECT_UP        0x01
#define UNIT_VECT_DOWN      0x02
#define UNIT_VECT_RIGHT     0x04
#define UNIT_VECT_LEFT      0x08
#define UNIT_VECT_NONE      0x00

//΂łĂ邩ǂp
#define DISC_VECT_NONE      0x00
#define DISC_VECT_UP        0x01
#define DISC_VECT_LOW       0x02
#define DISC_VECT_LEFT      0x04
#define DISC_VECT_RIGHT     0x08
#define DISC_VECT_LOW_LEFT  0x10
#define DISC_VECT_LOW_RIGHT 0x20
#define DISC_VECT_UP_LEFT   0x40
#define DISC_VECT_UP_RIGHT  0x80

//̓L[̃EFCg
#define KEY_RESPONSE        2

//Q[̐is
#define STATE_PUT_PLAYER    1
#define STATE_PUT_COMPUTER  2
#define STATE_CHK_PLAYER    3
#define STATE_CHK_COMPUTER  4
#define STATE_CHK_CLEAR     5
#define STATE_END           6

//At@x[^@Ŏgǂ݉
#define SEARCH_LEVEL        5
//---------------------------------------------------------------------------
//J[\
typedef struct {
	s16 x;
	s16 y;
	u16 cx;
	u16 cy;
	s16 MinX;		//ړ
	s16 MinY;
	s16 MaxX;
	s16 MaxY;
	u16 SprNo;		//XvCg
	u16 SprTile;
	u16 SprSize;
	u16 SprForm;
	u8  vect;		//ړp
} ST_CURSOR;

//Xe[W
typedef struct {
	u8   Turn;
	u8   Current;	//݂̎
	u16  Player;	//΂̎擾
	u16  Computer;	//΂̎擾
} ST_STAGE;

ST_CURSOR CURSOR;
ST_STAGE  STAGE;

//p}bv
u8 Map[MAP_CX][MAP_CY];
//---------------------------------------------------------------------------
u8 BG2Chr;
u8 BG2Map;
u8 BG2Col;

u8 BG3Chr;
u8 BG3Map;
u8 BG3Col;

u16 KeyCount;		//L[EFCgp
u16 KeyOld;			//L[EFCgp

u16 GameStateFlag;	//Q[̐is
//---------------------------------------------------------------------------
void Init();
void InitMap();
void InitCursor();

void ChkGameClear();
void ChkPlayer();
void ChkComputer();
void TurnPlayer();
void TurnComputer();

void PutMap(const u8 x, const u8 y, const u8 argDisc, bool argDrawFlag);
void FlipMap(const u8 argX, const u8 argY, const u8 argDisc, bool argDrawFlag);
void DrawMap(const u8 x, const u8 y);
void DrawMapAll();

void DrawBGMap(const u8 bgX, const u8 bgY, const u8 tileNo);
void DrawBGWinner();
void DrawBGScore();
void DrawSprCursor();

void Cursor();
void MoveCursor(ST_CURSOR* p);
bool ChkMoveCursor(ST_CURSOR* p);

bool ChkPass(const u8 argDisc);
u8   ChkMobility(const u8 argX, const u8 argY, const u8 argDisc);

s16  MaxLevel(const u8 level, s16 alpha, s16 beta);
s16  MinLevel(const u8 level, s16 alpha, s16 beta);

void MapSave(u8 backupMap[MAP_CX][MAP_CY]);
void MapLoad(u8 backupMap[MAP_CX][MAP_CY]);
s16  MapRating();
//---------------------------------------------------------------------------
int main(void)
{
	Init();
	InitMap();
	InitCursor();

	DrawBGScore();
	SetMode(MODE_0 | BG2_ENABLE | BG3_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);

	for(;;)
	{
		WaitForVsync();

		switch(GameStateFlag)
		{
			case STATE_CHK_PLAYER:   ChkPlayer();    break;
			case STATE_PUT_PLAYER:   TurnPlayer();   break;
			case STATE_CHK_COMPUTER: ChkComputer();  break;
			case STATE_PUT_COMPUTER: TurnComputer(); break;
			case STATE_CHK_CLEAR:    ChkGameClear(); break;
			case STATE_END:          goto End;       break;
			default: DebugMessage((u8*)"Err: Main GameStateFlag"); break;
		}
	}

End:
	DrawBGWinner();

	for(;;){}
	return 0;
}
//---------------------------------------------------------------------------
void Init()
{
	//EFCgRg[
	REG_WSCNT = CST_ROM0_1ST_3WAIT | CST_ROM0_2ND_1WAIT;

	//L[
	KeyInit();
	KeyCount = 0;
	KeyOld   = 0;

	//BG
	//BG2 \p
	//BG3 Q[p
	BG2Chr = 1;
	BG2Map = 0;
	BG2Col = 0;

	BG3Chr = 2;
	BG3Map = 1;
	BG3Col = 1;

	REG_BG2CNT = (BG_SIZEA_256_256    | BG_COLOR_16        |
				  BG_CHARBASE(BG2Chr) | BG_MAPBASE(BG2Map) | 0);

	REG_BG3CNT = (BG_SIZEA_256_256    | BG_COLOR_16        |
				  BG_CHARBASE(BG3Chr) | BG_MAPBASE(BG3Map) | 0);

	BGSetPal16  (BG2Col, (u16*)BGFont_palette);
	BGSetChrData(BG2Chr, (u16*)BGFont_data, BGFont_DATA_SIZE);

	BGSetPal16  (BG3Col, (u16*)&game_bg_pal_bin);
	BGSetChrData(BG3Chr, (u16*)&game_bg_img_bin, 2048);
	BGSetPalMap (BG3Map, 2048, 1);

	//Sprite
	SpriteInit();
	SpriteSetPal16((u16*)&game_spr_pal_bin);
	SpriteSetData ((u16*)&game_spr_img_bin, 2048);
}
//---------------------------------------------------------------------------
void InitMap()
{
	//tȌ
	GameStateFlag  = STATE_PUT_PLAYER;

	STAGE.Turn     = 1;
	STAGE.Current  = MAP_BLACK;		//݂̎
	STAGE.Player   = 2;
	STAGE.Computer = 2;

	//Map̏
	u8 x,y;
	for(y=0; y<MAP_CY; y++)
	{
		for(x=0; x<MAP_CX; x++)
		{
			Map[x][y] = MAP_EMPTY;
		}
	}

	Map[4][4] = MAP_WHITE; Map[5][4] = MAP_BLACK;
	Map[4][5] = MAP_BLACK; Map[5][5] = MAP_WHITE;

	//Oǂ̍쐬
	for(x=0; x<MAP_CX; x++)
	{
		Map[x][0       ] = MAP_BLOCK;
		Map[x][MAP_CX-1] = MAP_BLOCK;
	}
	for(y=0; y<MAP_CY; y++)
	{
		Map[0       ][y] = MAP_BLOCK;
		Map[MAP_CY-1][y] = MAP_BLOCK;
	}

	//Map\
	DrawMapAll();
}
//---------------------------------------------------------------------------
void InitCursor()
{
	CURSOR.x       = 16;
	CURSOR.y       = 16;
	CURSOR.cx      = 16;
	CURSOR.cy      = 16;

	//ړ̐ݒ
	CURSOR.MinX    = BG_CX;
	CURSOR.MaxX    = BG_CX * (MAP_CX-1);
	CURSOR.MinY    = BG_CY;
	CURSOR.MaxY    = BG_CY * (MAP_CY-1);
	CURSOR.vect    = UNIT_VECT_NONE;

	CURSOR.SprNo   = 0;
	CURSOR.SprTile = 0;
	CURSOR.SprSize = SP_SIZE_16;
	CURSOR.SprForm = SP_SQUARE;

	SpriteSetSize(CURSOR.SprNo, CURSOR.SprSize, CURSOR.SprForm, SP_COLOR_16);
	SpriteSetChr (CURSOR.SprNo, CURSOR.SprTile);
	SpriteMove   (CURSOR.SprNo, CURSOR.x, CURSOR.y);

	DrawSprCursor();
}
//---------------------------------------------------------------------------
void DrawMapAll()
{
	u8 x,y;
	
	for(y=1; y<MAP_CY-1; y++)
	{
		for(x=1; x<MAP_CX-1; x++)
		{
			DrawMap(x,y);
		}
	}
}
//---------------------------------------------------------------------------
void DrawMap(const u8 x, const u8 y)
{
	u8 chr    = Map[x][y];
	u8 bgX    = x * 2;		//MapWBGWɕϊ
	u8 bgY    = y * 2;
	u8 tileNo = 0;

	switch(chr)
	{
		case MAP_EMPTY: tileNo = BG_TILE_EMPTY; break;
		case MAP_WHITE: tileNo = BG_TILE_WHITE; break;
		case MAP_BLACK: tileNo = BG_TILE_BLACK; break;
		default: DebugMessage((u8*)"Err: DrawMap Tile"); break;
	}

	DrawBGMap(bgX, bgY, tileNo);
}
//---------------------------------------------------------------------------
void DrawBGMap(const u8 bgX, const u8 bgY, const u8 tileNo)
{
	BGDrawChar(BG3Map, bgX+0, bgY+0, BG3Col, tileNo+0 );
	BGDrawChar(BG3Map, bgX+1, bgY+0, BG3Col, tileNo+1 );
	BGDrawChar(BG3Map, bgX+0, bgY+1, BG3Col, tileNo+32);
	BGDrawChar(BG3Map, bgX+1, bgY+1, BG3Col, tileNo+33);
}
//---------------------------------------------------------------------------
void DrawBGWinner()
{
	if(STAGE.Player == STAGE.Computer)     BGDrawStr(BG2Map, 2, 18, (u8*)"Draw!");
	else if(STAGE.Player > STAGE.Computer) BGDrawStr(BG2Map, 2, 18, (u8*)"Player   Win!");
	else if(STAGE.Player < STAGE.Computer) BGDrawStr(BG2Map, 2, 18, (u8*)"Computer Win!");
}
//---------------------------------------------------------------------------
void DrawBGScore()
{
	char msg[40];
	bprintf(msg, "Turn  :%2d", STAGE.Turn);
	BGDrawStr(BG2Map, 19, 2, (u8*)msg);
	bprintf(msg, "Player:%2d", STAGE.Player);
	BGDrawStr(BG2Map, 19, 4, (u8*)msg);
	bprintf(msg, "Com   :%2d", STAGE.Computer);
	BGDrawStr(BG2Map, 19, 5, (u8*)msg);
}
//---------------------------------------------------------------------------
void DrawSprCursor()
{
	SpriteMove(CURSOR.SprNo, CURSOR.x, CURSOR.y);
}
//---------------------------------------------------------------------------
void PutMap(const u8 x, const u8 y, const u8 argDisc, bool argDrawFlag)
{
	//fBXvCɕ\邩̔
	if(argDrawFlag == TRUE)
	{
		if(Map[x][y] == MAP_WHITE) STAGE.Computer--;
		if(Map[x][y] == MAP_BLACK) STAGE.Player--;
		if(argDisc   == MAP_WHITE) STAGE.Computer++;
		if(argDisc   == MAP_BLACK) STAGE.Player++;

		Map[x][y] = argDisc;
		DrawMap(x, y);
	}
	else
	{
		Map[x][y] = argDisc;
	}
}
//---------------------------------------------------------------------------
//J[\̐U镑
void Cursor()
{
	u8 Key = KeyInput();

	CURSOR.vect = UNIT_VECT_NONE;

	//L[X|X(TODO ꃉCuj
	if(Key == 0x00 && KeyOld != Key)
	{
		KeyCount = 0;
		KeyOld   = Key;
		return;
	}
	if(KeyCount++ <= KEY_RESPONSE)
	{
		return;
	}

	KeyCount = 0;

	if(Key & KEY_UP   ) CURSOR.vect |= UNIT_VECT_UP;
	if(Key & KEY_DOWN ) CURSOR.vect |= UNIT_VECT_DOWN;
	if(Key & KEY_LEFT ) CURSOR.vect |= UNIT_VECT_LEFT;
	if(Key & KEY_RIGHT) CURSOR.vect |= UNIT_VECT_RIGHT;
	if(Key & KEY_A    )
	{
		u8 x = CURSOR.x / CURSOR.cx;
		u8 y = CURSOR.y / CURSOR.cy;

		//J[\Ŏw肵Wɐ΂͂邩`FbN
		if( ChkMobility(x, y, MAP_BLACK) != 0 )
		{
			FlipMap(x, y, MAP_BLACK, TRUE);	//ӂ̐΂̐FύX
			PutMap (x, y, MAP_BLACK, TRUE);	//wWɐ΂u
			GameStateFlag = STATE_CHK_CLEAR;
			STAGE.Current = MAP_WHITE;
		}
	}

	if( ChkMoveCursor(&CURSOR) == TRUE )
	{
		MoveCursor(&CURSOR);
		DrawSprCursor();
	}
}
//---------------------------------------------------------------------------
//ړ͈ړ\`FbN
u8 ChkMoveCursor(ST_CURSOR* p)
{
	s16 tx1 = p->x;
	s16 ty1 = p->y;

	if(p->vect & UNIT_VECT_UP   ) ty1-=p->cy;
	if(p->vect & UNIT_VECT_DOWN ) ty1+=p->cy;
	if(p->vect & UNIT_VECT_LEFT ) tx1-=p->cx;
	if(p->vect & UNIT_VECT_RIGHT) tx1+=p->cx;

	if(tx1< CURSOR.MinX || ty1< CURSOR.MinY) return FALSE;
	if(tx1>=CURSOR.MaxX || ty1>=CURSOR.MaxY) return FALSE;

	return TRUE;
}
//---------------------------------------------------------------------------
//ړ
void MoveCursor(ST_CURSOR* p)
{
	if(p->vect & UNIT_VECT_UP   ) p->y-=p->cy;
	if(p->vect & UNIT_VECT_DOWN ) p->y+=p->cy;
	if(p->vect & UNIT_VECT_LEFT ) p->x-=p->cx;
	if(p->vect & UNIT_VECT_RIGHT) p->x+=p->cx;
}
//---------------------------------------------------------------------------
//w肵Wɐ΂u邩`FbN܂ireturnl͕tOj
u8 ChkMobility(const u8 argX, const u8 argY, const u8 argDisc)
{
	if(Map[argX][argY] != MAP_EMPTY)
	{
		return DISC_VECT_NONE;
	}

	u8 opposeDisc = (argDisc == MAP_BLACK) ? MAP_WHITE : MAP_BLACK;		//G̐Fi[
	u8 retFlag    = 0x00;	//߂li[
	u8 x, y;

	//
	if(Map[argX][argY-1] == opposeDisc)
	{
		x = argX;
		y = argY - 2;
		while(Map[x][y] == opposeDisc) y--;
		if   (Map[x][y] ==    argDisc) retFlag |= DISC_VECT_UP;
	}
	//
	if(Map[argX][argY+1] == opposeDisc)
	{
		x = argX;
		y = argY + 2;
		while(Map[x][y] == opposeDisc) y++;
		if   (Map[x][y] ==    argDisc) retFlag |= DISC_VECT_LOW;
	}
	//
	if(Map[argX-1][argY] == opposeDisc)
	{
		x = argX - 2;
		y = argY;
		while(Map[x][y] == opposeDisc) x--;
		if   (Map[x][y] ==    argDisc) retFlag |=  DISC_VECT_LEFT;
	}
	//E
	if(Map[argX+1][argY] == opposeDisc)
	{
		x = argX + 2;
		y = argY;
		while(Map[x][y] == opposeDisc) x++;
		if   (Map[x][y] ==    argDisc) retFlag |= DISC_VECT_RIGHT;
	}
	//E
	if(Map[argX+1][argY-1] == opposeDisc)
	{
		x = argX + 2;
		y = argY - 2;
		while(Map[x][y] == opposeDisc) { x++; y--; }
		if   (Map[x][y] ==    argDisc) retFlag |= DISC_VECT_UP_RIGHT;
	}
	//
	if(Map[argX-1][argY-1] == opposeDisc)
	{
		x = argX - 2;
		y = argY - 2;
		while(Map[x][y] == opposeDisc) { x--; y--; }
		if   (Map[x][y] ==    argDisc) retFlag |= DISC_VECT_UP_LEFT;
	}
	//
	if(Map[argX-1][argY+1] == opposeDisc)
	{
		x = argX - 2;
		y = argY + 2;
		while(Map[x][y] == opposeDisc) { x--; y++; }
		if   (Map[x][y] ==    argDisc) retFlag |= DISC_VECT_LOW_LEFT;
	}
	//E
	if(Map[argX+1][argY+1] == opposeDisc)
	{
		x = argX + 2;
		y = argY + 2;
		while(Map[x][y] == opposeDisc) { x++; y++; }
		if   (Map[x][y] ==    argDisc) retFlag |= DISC_VECT_LOW_RIGHT;
	}

	return retFlag;
}
//---------------------------------------------------------------------------
//w肵W̎ӂ𔽓]܂
void FlipMap(const u8 argX, const u8 argY, const u8 argDisc, bool argDrawFlag)
{
	u8 opposeDisc = (argDisc == MAP_BLACK) ? MAP_WHITE : MAP_BLACK;		//G̐Fi[
	u8 vect       = ChkMobility(argX, argY, argDisc);
	u8 x, y;

	//
	if(vect & DISC_VECT_UP)
	{
		x = argX;
		y = argY - 1;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			y--;
		}
	}
	//
	if(vect & DISC_VECT_LOW)
	{
		x = argX;
		y = argY + 1;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			y++;
		}
	}
	//
	if(vect & DISC_VECT_LEFT)
	{
		x = argX - 1;
		y = argY;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			x--;
		}
	}
	//E
	if(vect & DISC_VECT_RIGHT)
	{
		x = argX + 1;
		y = argY;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			x++;
		}
	}
	//E
	if(vect & DISC_VECT_UP_RIGHT)
	{
		x = argX + 1;
		y = argY - 1;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			x++; y--;
		}
	}
	//
	if(vect & DISC_VECT_UP_LEFT)
	{
		x = argX - 1;
		y = argY - 1;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			x--; y--;
		}
	}
	//
	if(vect & DISC_VECT_LOW_LEFT)
	{
		x = argX - 1;
		y = argY + 1;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			x--; y++;
		}
	}
	//E
	if(vect & DISC_VECT_LOW_RIGHT)
	{
		x = argX + 1;
		y = argY + 1;
		while(Map[x][y] == opposeDisc)
		{
			PutMap(x, y, argDisc, argDrawFlag);
			x++; y++;
		}
	}
}
//---------------------------------------------------------------------------
//Ԃ̃pX`FbN(߂l TRUE:uꏊ FALSE:uꏊȂ)
bool ChkPass(const u8 argDisc)
{
	u8 x, y;
	for(y=1; y<MAP_CY-1; y++)
	{
		for(x=1; x<MAP_CX-1; x++)
		{
			if( ChkMobility(x,y,argDisc) != 0 )
			{
				return TRUE;
			}
		}
	}

	return FALSE;
}
//---------------------------------------------------------------------------
void ChkPlayer()
{
	//pX`FbN
	if( ChkPass(MAP_BLACK) == TRUE ) GameStateFlag = STATE_PUT_PLAYER;
	else                             GameStateFlag = STATE_CHK_COMPUTER;
}
//---------------------------------------------------------------------------
void ChkComputer()
{
	//pX`FbN
	if( ChkPass(MAP_WHITE) == TRUE ) GameStateFlag = STATE_PUT_COMPUTER;
	else                             GameStateFlag = STATE_CHK_PLAYER;
}
//---------------------------------------------------------------------------
void TurnPlayer()
{
	Cursor();	//L[͂sȂ
}
//---------------------------------------------------------------------------

//Ֆʕ]Ɏgp܂
s16 AIMap[MAP_CX][MAP_CY] = {
	{  0,   0,   0,   0,   0,   0,   0,    0,   0,  0},
	{  0, 120, -20,  20,   5,   5,  20,  -20, 120,  0},
	{  0, -20, -40,  -5,  -5,  -5,  -5,  -40, -20,  0},
	{  0,  20,  -5,  15,   3,   3,  15,   -5,  20,  0},
	{  0,   5,  -5,   3,   3,   3,   3,   -5,   5,  0},
	{  0,   5,  -5,   3,   3,   3,   3,   -5,   5,  0},
	{  0,  20,  -5,  15,   3,   3,  15,   -5,  20,  0},
	{  0, -20, -40,  -5,  -5,  -5,  -5,  -40, -20,  0},
	{  0, 120, -20,  20,   5,   5,  20,  -20, 120,  0},
	{  0,   0,   0,   0,   0,   0,   0,    0,   0,  0},
};
//-------------------------------------
//Rs[^̎
void TurnComputer()
{
	//߂Ălu8TCY0-3rbgYWA7-4rbgXW
	u8 result = MaxLevel(SEARCH_LEVEL, -5000, 5000);
	u8 x      = (result>>4) & 0x0f;
	u8 y      = result & 0x0f;

	FlipMap(x, y, MAP_WHITE, TRUE);	//ӂ̐΂̐FύX
	PutMap (x, y, MAP_WHITE, TRUE);	//wWɐ΂u
	GameStateFlag = STATE_CHK_CLEAR;
	STAGE.Current = MAP_BLACK;
}
//-------------------------------------
//Ֆʂ̕]܂
s16 MapRating()
{
    s16 retScore = 0;		//]l(߂l)
	u8  x, y;

	for(y=1; y<MAP_CY-1; y++)
	{
		for(x=1; x<MAP_CX-1; x++)
		{
			if(Map[x][y] == MAP_WHITE) retScore -= AIMap[x][y];
			if(Map[x][y] == MAP_BLACK) retScore += AIMap[x][y];
		}
	}
	return -retScore;
}
//-------------------------------------
//At@x[^@
s16 MaxLevel(const u8 level, s16 alpha, s16 beta)
{
	//TIĔՖʂ̕]lԂ܂
	if(level == 0)
	{
	    return MapRating();
	}

	u8  backupMap[MAP_CX][MAP_CY];	//Ֆʂ̃obNAbvp
	s16 childScore;					//qm[h̕]l
	s16 score = -5000;				//m[h̕]l
	u8  bestX = 0;
	u8  bestY = 0;
	u8  x, y;

	for(y=1; y<MAP_CY-1; y++)
	{
		for(x=1; x<MAP_CX-1; x++)
		{
			if( ChkMobility(x,y,MAP_WHITE) != 0 )
			{
				MapSave(backupMap);
				FlipMap(x, y, MAP_WHITE, FALSE);	//ӂ̐΂̐FύX
				PutMap (x, y, MAP_WHITE, FALSE);	//wWɐ΂u
				childScore = MinLevel(level - 1, alpha, beta);
				MapLoad(backupMap);

				//ő̕]lI
				if(childScore > score)
				{
					alpha = childScore;
					score = childScore;
					bestX = x;
					bestY = y;
				}
				//x[^Jbg
				if(score > beta)
				{
					return score;
				}
			}
	    }
	}

	if(level == SEARCH_LEVEL)
	{
		return ((bestX<<4) & 0xf0) + (bestY & 0x0f);
	}
	return score;
}
//-------------------------------------
s16 MinLevel(const u8 level, s16 alpha, s16 beta)
{
	//TIĔՖʂ̕]lԂ܂
	if(level == 0)
	{
	    return MapRating();
	}

	u8  backupMap[MAP_CX][MAP_CY];	//Ֆʂ̃obNAbvp
	s16 childScore;					//qm[h̕]l
	s16 score = 5000;				//m[h̕]l
	u8  bestX = 0;
	u8  bestY = 0;
	u8  x, y;

	for(y=1; y<MAP_CY-1; y++)
	{
		for(x=1; x<MAP_CX-1; x++)
		{
			if( ChkMobility(x,y,MAP_BLACK) != 0 )
			{
				MapSave(backupMap);
				FlipMap(x, y, MAP_BLACK, FALSE);	//ӂ̐΂̐FύX
				PutMap (x, y, MAP_BLACK, FALSE);	//wWɐ΂u
				childScore = MaxLevel(level - 1, alpha, beta);
				MapLoad(backupMap);

				//ŏ̕]lI
				if(childScore < score)
				{
					beta  = childScore;
					score = childScore;
					bestX = x;
					bestY = y;
				}
				//At@Jbg
				if(score < beta)
				{
					return score;
				}
			}
	    }
	}

	if(level == SEARCH_LEVEL)
	{
		return ((bestX<<4) & 0xf0) + (bestY & 0x0f);
	}
	return score;
}
//---------------------------------------------------------------------------
void ChkGameClear()
{
	// 60ɒBĂȂꍇ
	if(STAGE.Turn < 60)
	{
		goto goNext;
	}

	//łꏊꍇ
//	if(STAGE.Player + STAGE.Computer < 64)
//	{
//		goto goNext;
//	}

	// łĂꏊꍇ
	if(ChkPass(MAP_BLACK) != 0 || ChkPass(MAP_WHITE) != 0)
	{
		goto goNext;
	}

	//Q[̏I
	GameStateFlag = STATE_END;
	DrawBGScore();
	return;

//̎Ԃ̏
goNext:

	if(STAGE.Current == MAP_WHITE)
	{
		GameStateFlag = STATE_CHK_COMPUTER;
	}
	else
	{
		GameStateFlag = STATE_CHK_PLAYER;
	}
	STAGE.Turn++;
	DrawBGScore();
	return;
}
//---------------------------------------------------------------------------
//Ֆʂۑ܂
void MapSave(u8 backupMap[MAP_CX][MAP_CY])
{
	u8 x,y;

	for(y=0; y<MAP_CY; y++)
	{
		for(x=0; x<MAP_CX; x++)
		{
			backupMap[x][y] = Map[x][y];
		}
	}
}
//---------------------------------------------------------------------------
//Ֆʂ߂܂
void MapLoad(u8 backupMap[MAP_CX][MAP_CY])
{
	u8 x,y;

	for(y=0; y<MAP_CY; y++)
	{
		for(x=0; x<MAP_CX; x++)
		{
			Map[x][y] = backupMap[x][y];
		}
	}
}
